@@ -3,11 +3,14 @@ use warnings;
 
 package Dancer2::Plugin::Auth::HTTP::Basic::DWIW;
 # ABSTRACT: HTTP Basic authentication plugin for Dancer2 that does what I want.
-$Dancer2::Plugin::Auth::HTTP::Basic::DWIW::VERSION = '0.05';
+$Dancer2::Plugin::Auth::HTTP::Basic::DWIW::VERSION = '0.06';
 use MIME::Base64;
 use Dancer2::Plugin;
 
-our $CHECK_LOGIN_HANDLER = undef;
+our $HANDLERS = {
+    check_login => undef,
+    no_auth     => undef,
+};
 
 register http_basic_auth => sub {
     my ($dsl, $stuff, $sub, @other_stuff) = @_;
@@ -27,15 +30,17 @@ register http_basic_auth => sub {
 
             $username || $password || die \401;
 
-            if (ref($CHECK_LOGIN_HANDLER) eq 'CODE') {
-                my $check_result = eval { $CHECK_LOGIN_HANDLER->($username, $password); };
+            if(my $handler = $HANDLERS->{check_login}) {
+                if(ref($handler) eq 'CODE') {
+                    my $check_result = eval { $handler->($username, $password); };
 
-                if($@) {
-                    die \500;
-                }
+                    if($@) {
+                        die \500;
+                    }
 
-                if(!$check_result) {
-                    die \401;
+                    if(!$check_result) {
+                        die \401;
+                    }
                 }
             }
         };
@@ -48,6 +53,13 @@ register http_basic_auth => sub {
 
             $dsl->header('WWW-Authenticate' => 'Basic realm="' . $realm . '"');
             $dsl->status($error_code);
+
+            if(my $handler = $HANDLERS->{no_auth}) {
+                if(ref($handler) eq 'CODE') {
+                    return $handler->();
+                }
+            }
+
             return;
         }
     };
@@ -69,7 +81,14 @@ register http_basic_auth_login => sub {
 
 register http_basic_auth_set_check_handler => sub {
     my ($dsl, $handler) = @_;
-    $CHECK_LOGIN_HANDLER = $handler;
+
+    warn 'This is deprecated! Please use http_basic_auth_handler check_login => sub {}';
+    $dsl->http_basic_auth_handler(check_login => $handler);
+};
+
+register http_basic_auth_handler => sub {
+    my ($dsl, $name, $handler) = @_;
+    $HANDLERS->{$name} = $handler;
 };
 
 register_plugin for_versions => [2];
@@ -87,7 +106,7 @@ Dancer2::Plugin::Auth::HTTP::Basic::DWIW - HTTP Basic authentication plugin for
 
 =head1 VERSION
 
-version 0.05
+version 0.06
 
 =head1 SYNOPSIS
 
@@ -96,13 +115,17 @@ version 0.05
     use Dancer2;
     use Dancer2::Plugin::Auth::HTTP::Basic::DWIW;
 
-    http_basic_auth_set_check_handler sub {
+    http_basic_auth_handler check_login => sub {
         my ( $user, $pass ) = @_;
 
         # you probably want to check the user in a better way
         return $user eq 'test' && $pass eq 'bla';
     };
 
+    http_basic_auth_handler no_auth => sub {
+        template 'auth_error';
+    };
+
     get '/' => http_basic_auth required => sub {
         my ( $user, $pass ) = http_basic_auth_login;
 
@@ -152,11 +175,11 @@ After installation you can find documentation for this module with the perldoc c
 
 =head1 AUTHOR
 
-Moritz Grosch (LittleFox) <littlefox@fsfe.org>
+Mara Sophie Grosch (LittleFox) <littlefox@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2017 by Moritz Grosch (LittleFox).
+This software is copyright (c) 2019 by Mara Sophie Grosch (LittleFox).
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
